@@ -1,4 +1,4 @@
-/*  $Revision: 1.51 $
+/*  $Revision: 1.52 $
 **
 **  Article-processing.
 */
@@ -58,6 +58,7 @@ typedef struct _ARTHEADER {
     char		*Value;
     int			Length;			/* Length of Value	*/
     int			Found;
+    BOOL		Allocated;
 } ARTHEADER;
 
 
@@ -147,7 +148,9 @@ STATIC ARTHEADER	ARTheaders[] = {
     {	"Also-Control",		HTstd },
 #define _alsocontrol		15
     {	"References",		HTstd },
+#define _references		16
     {	"Xref",			HTobs },
+#define _xref			17
     {	"Date-Received",	HTobs },
     {	"Posted",		HTobs },
     {	"Posting-Version",	HTobs },
@@ -301,7 +304,9 @@ ARTsetup()
     /* Allocate space in the header table. */
     for (hp = ARTheaders; hp < ENDOF(ARTheaders); hp++) {
 	hp->Size = strlen(hp->Name);
-	if (hp->Value == NULL && hp->Type != HTobs)
+	hp->Allocated = hp->Value == NULL && hp->Type != HTobs
+			&& hp != &ARTheaders[_bytes];
+	if (hp->Allocated)
 	    hp->Value = NEW(char, MAXHEADERSIZE + 1);
     }
 
@@ -322,6 +327,39 @@ ARTsetup()
 }
 
 
+STATIC void
+ARTfreetree(tp)
+    TREE	*tp;
+{
+    TREE	*next;
+
+    for ( ; tp != NULL; tp = next) {
+	if (tp->Before)
+	    ARTfreetree(tp->Before);
+	next = tp->After;
+	DISPOSE(tp);
+    }
+}
+
+
+void
+ARTclose()
+{
+    register ARTHEADER	*hp;
+
+    /* Free space in the header table. */
+    for (hp = ARTheaders; hp < ENDOF(ARTheaders); hp++)
+	if (hp->Allocated)
+	    DISPOSE(hp->Value);
+
+    if (ARTfields != NULL) {
+	DISPOSE(ARTfields);
+	ARTfields = NULL;
+    }
+    ARTfreetree(ARTheadertree);
+}
+
+
 /*
 **  Read in a file, return a pointer to static space that is reused.
 */
@@ -550,12 +588,17 @@ ARTwrite(name, Article, Data, CrossPosted)
 	(vp++)->iov_len  = i;
 	size += i;
 	/* Install in header table; STRLEN("Lines: ") == 7. */
-	HDR(_lines) = Data->Lines + 7;
+	(void)strcpy(ARTheaders[_lines].Value, Data->Lines + 7);
 	ARTheaders[_lines].Length = i - 7;
 	ARTheaders[_lines].Found = 1;
     }
 
     if (CrossPosted) {
+	/* Install in header table; STRLEN("Xref: ") == 6. */
+	HDR(_xref) = Xref.Data + 6;
+	ARTheaders[_xref].Length = Xref.Used - 6;
+	ARTheaders[_xref].Found = 1;
+
 	vp->iov_base = Xref.Data;
 	vp->iov_len  = Xref.Used;
 	size += (vp++)->iov_len;
@@ -853,7 +896,7 @@ ARTclean(Article, Data)
     Data->Size[0] = '0';
     Data->Size[1] = '\0';
     for (hp = ARTheaders; hp < ENDOF(ARTheaders); hp++) {
-	if (hp->Value)
+	if (hp->Value && hp->Type != HTobs)
 	    *hp->Value = '\0';
 	hp->Found = 0;
     }
@@ -1334,7 +1377,7 @@ ARTsortfordisk()
 **  group were created.
 */
 STATIC void
-ARTsendnewgroup(name)
+ARTsendthegroup(name)
     register char	*name;
 {
     register SITE	*sp;
@@ -1497,7 +1540,7 @@ ARTpropagate(Data, hops, hopcount, list)
 	    /* Too big for the site. */
 	    continue;
 
-	if (ListHas(hops, sp->Name)
+	if ((!sp->IgnorePath && ListHas(hops, sp->Name))
 	 || (sp->Hops && hopcount > sp->Hops)
 	 || (sp->Groupcount && Groupcount > sp->Groupcount))
 	    /* Site already saw the article; path too long; or too much
@@ -1633,7 +1676,7 @@ ARTpost(cp, Replic, ihave)
     ARTDATA		Data;
     BOOL		Approved;
     BOOL		Accepted;
-    BOOL		IsNewgroup;
+    BOOL		LikeNewgroup;
     BOOL		CrossPosted;
     BOOL		ToGroup;
     BOOL		GroupMissing;
@@ -1662,6 +1705,8 @@ ARTpost(cp, Replic, ihave)
     hops = ARTparsepath(HDR(_path), &hopcount);
 #if	defined(DO_IPADDR_LOG)
     Data.Feedsite = RChostname(cp);
+    if (Data.Feedsite == NULL)
+	Data.Feedsite = CHANname(cp);
 #else
     Data.Feedsite = hops && hops[0] ? hops[0] : CHANname(cp);
 #endif	/* defined(DO_IPADDRLOG) */
@@ -1739,7 +1784,7 @@ ARTpost(cp, Replic, ihave)
 	ControlHeader = _alsocontrol;
     else {
 	ControlHeader = -1;
-	IsNewgroup = FALSE;
+	LikeNewgroup = FALSE;
     }
     if (ControlHeader >= 0) {
 	/* Nip off the first word into lowercase. */
@@ -1749,7 +1794,8 @@ ARTpost(cp, Replic, ihave)
 	    if (CTYPE(isupper, *p))
 		*p = tolower(*p);
 	*p = '\0';
-	IsNewgroup = EQ(ControlWord, "newgroup");
+	LikeNewgroup = EQ(ControlWord, "newgroup")
+		    || EQ(ControlWord, "rmgroup");
 
 	/* Control messages to "foo.ctl" are treated as if they were
 	 * posted to "foo".  I should probably apologize for all the
@@ -1776,12 +1822,12 @@ ARTpost(cp, Replic, ihave)
 	    continue;
 	if ((ngp = NGfind(p)) == NULL) {
 	    GroupMissing = TRUE;
-	    if (IsNewgroup && Approved) {
-		/* Newgroup being sent to a group that doesn't exist.
-		 * Assume it is being sent to the group being created,
-		 * and send the group to all sites that would get the
-		 * group if it were created. */
-		ARTsendnewgroup(*groups);
+	    if (LikeNewgroup && Approved) {
+		/* Newgroup/rmgroup being sent to a group that doesn't
+		 * exist.  Assume it is being sent to the group being
+		 * created or removed, nd send the group to all sites that
+		 * would or would have had the group if it were created. */
+		ARTsendthegroup(*groups);
 		Accepted = TRUE;
 	    }
 
@@ -1872,7 +1918,7 @@ ARTpost(cp, Replic, ihave)
 		DISPOSE(distributions);
 	    ARTreject(buff, article);
 	    return buff;
-#endif	/* defined(DONT_WANT_TRASH) */
+#else
 	    /* if !GroupMissing, then all the groups the article was posted
 	     * to have a flag of "x" in our active file, and therefore
 	     * we should throw the article away:  if you have define
@@ -1884,6 +1930,7 @@ ARTpost(cp, Replic, ihave)
 		ARTreject(buff, article);
 		return buff;
 	    }
+#endif	/* defined(DONT_WANT_TRASH) */
 	}
 	ngp = NGfind(ARTjnk);
 	*ngptr++ = ngp;
