@@ -1,4 +1,4 @@
-/*  $Revision: 1.14 $
+/*  $Revision: 1.17 $
 **
 **  Read batchfiles on standard input and spew out batches.
 */
@@ -25,6 +25,7 @@
 **  Global variables.
 */
 STATIC BOOL	BATCHopen;
+STATIC BOOL	STATprint;
 STATIC double	STATbegin;
 STATIC double	STATend;
 STATIC char	*Host;
@@ -109,10 +110,13 @@ RequeueAndExit(Cookie, line, BytesInArt)
 	usertime = 0;
 	systime = 0;
     }
-    (void)printf(LINE1, Host, usertime, systime, STATend - STATbegin);
-    (void)printf("\n");
-    (void)printf(LINE2, Host, BATCHcount, ArtsWritten, BytesWritten);
-    (void)printf("\n");
+
+    if (STATprint) {
+	(void)printf(LINE1, Host, usertime, systime, STATend - STATbegin);
+	(void)printf("\n");
+	(void)printf(LINE2, Host, BATCHcount, ArtsWritten, BytesWritten);
+	(void)printf("\n");
+    }
 
     (void)openlog("batcher", L_OPENLOG_FLAGS | LOG_PID, LOG_INN_PROG);
     syslog(L_NOTICE, LINE1, Host, usertime, systime, STATend - STATbegin);
@@ -143,6 +147,7 @@ RequeueAndExit(Cookie, line, BytesInArt)
     }
 
     /* If we can back up to where the batch started, do so. */
+    i = 0;
     if (Cookie != -1 && fseek(stdin, Cookie, SEEK_SET) == -1) {
 	(void)fprintf(stderr, "batcher %s cant seek %s\n",
 		Host, strerror(errno));
@@ -235,9 +240,10 @@ main(ac, av)
     /* Set defaults. */
     AltSpool = NULL;
     Redirect = TRUE;
+    (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "a:A:b:B:i:N:p:rs:S:")) != EOF)
+    while ((i = getopt(ac, av, "a:A:b:B:i:N:p:rs:S:v")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -272,6 +278,9 @@ main(ac, av)
 	case 'S':
 	    AltSpool = optarg;
 	    break;
+	case 'v':
+	    STATprint = TRUE;
+	    break;
 	}
     if (MaxArts && ArtsInBatch == 0)
 	ArtsInBatch = MaxArts;
@@ -316,6 +325,7 @@ main(ac, av)
     (void)signal(SIGHUP, CATCHinterrupt);
     (void)signal(SIGINT, CATCHinterrupt);
     (void)signal(SIGTERM, CATCHinterrupt);
+    /* (void)signal(SIGPIPE, CATCHinterrupt); */
     (void)GetTimeInfo(&Now);
     STATbegin = TIMEINFOasDOUBLE(Now);
 
@@ -323,7 +333,7 @@ main(ac, av)
     while (fgets(line, sizeof line, stdin) != NULL) {
 	/* Record line length in case we do an ftell. Not portable to
 	 * systems with non-Unix file formats. */
-	length = strlen(line) + 1;
+	length = strlen(line);
 
 	/* Get lines like "name size" */
 	if ((p = strchr(line, '\n')) == NULL) {
@@ -335,7 +345,10 @@ main(ac, av)
 	    continue;
 	if ((p = strchr(line, ' ')) != NULL) {
 	    *p++ = '\0';
-	    BytesInArt = atol(p);
+	    if ((BytesInArt = atol(p)) == 0) {
+		/* probably a "Wnm"-style (NNTP) batch file, work around it */
+		BytesInArt = -1;
+	    }
 	}
 	else
 	    BytesInArt = -1;
