@@ -1,4 +1,4 @@
-/*  $Revision: 1.29 $
+/*  $Revision: 1.30 $
 **
 **  I/O channel (and buffer) processing.
 */
@@ -63,9 +63,12 @@ CHANsetup(i)
     if (CHANtable)
 	DISPOSE(CHANtable);
     CHANtablesize = i;
+    CHANtable = NEW(CHANNEL, CHANtablesize);
+    (void)memset((POINTER)CHANtable, 0,
+	    (SIZE_T)(CHANtablesize * sizeof *CHANtable));
     CHANnull.NextLog = CHANNEL_INACTIVE_TIME;
     CHANnull.Address.s_addr = MyAddress.s_addr;
-    for (cp = CHANtable = NEW(CHANNEL, CHANtablesize); --i >= 0; cp++)
+    for (cp = CHANtable; --i >= 0; cp++)
 	*cp = CHANnull;
 }
 
