@@ -1,5 +1,5 @@
 #! /bin/sh
-##  $Revision: 1.4 $
+##  $Revision: 1.5 $
 ##
 ##  Prepare a manpage for installation, and install it.  Usage:
 ##	putman <style> "<installitflags>" <source> <dest-dir>
@@ -25,7 +25,23 @@ XSOURCE)
 XNROFF-PACK)
     T=${TMPDIR-/tmp}/man$$
     nroff -man ${SRC} >$T
-    /bin/sh ../installit.sh ${FLAGS} $T ${DEST} && pack ${DEST}/${SRC}
+    /bin/sh ../installit.sh ${FLAGS} $T ${DEST} && pack ${DEST}
+    rm -f $T
+    exit
+    ;;
+XNROFF-PACK-SCO)
+    T=${TMPDIR-/tmp}/man$$
+    nroff -man ${SRC} >$T
+    DEST2=`echo ${DEST} | sed -e 's/\..$/.INN/'`
+    /bin/sh ../installit.sh ${FLAGS} $T ${DEST2} && pack ${DEST2}
+    rm -f $T
+    exit
+    ;;
+XNROFF-NET2)
+    T=${TMPDIR-/tmp}/man$$
+    nroff -man ${SRC} >$T
+    DEST2=`echo ${DEST} | sed -e 's/\..$/.0/'`
+    /bin/sh ../installit.sh ${FLAGS} $T ${DEST2}
     rm -f $T
     exit
     ;;
