@@ -1,4 +1,4 @@
-/*  $Revision: 1.13 $
+/*  $Revision: 1.17 $
 **
 **  Expire news articles.
 */
@@ -58,12 +58,20 @@ typedef struct _NGHASH {
 */
 #define MAGIC_TIME	49710.
 
+typedef struct _BADGROUP {
+    struct _BADGROUP	*Next;
+    char		*Name;
+    BOOL		HasDirectory;
+} BADGROUP;
+
+STATIC BADGROUP		*EXPbadgroups;
 STATIC BOOL		EXPlinks;
 STATIC BOOL		EXPquiet;
 STATIC BOOL		EXPsizing;
 STATIC BOOL		EXPtracing;
 STATIC BOOL		EXPusepost;
 STATIC char		EXPnewslib[] = _PATH_NEWSLIB;
+STATIC char		ACTIVE[] = _PATH_ACTIVE;
 STATIC char		SPOOL[] = _PATH_SPOOL;
 STATIC int		nGroups;
 STATIC FILE		*EXPunlinkfile;
@@ -76,6 +84,7 @@ STATIC time_t		EXPremember;
 STATIC time_t		Now;
 
 /* Statistics; for -v flag. */
+STATIC char		*EXPgraph;
 STATIC int		EXPverbose;
 STATIC long		EXPprocessed;
 STATIC long		EXPunlinked;
@@ -93,7 +102,7 @@ extern double		atof();
 /*
 **  Hash a newsgroup and see if we get it.
 */
-NEWSGROUP *
+STATIC NEWSGROUP *
 NGfind(Name)
     char		*Name;
 {
@@ -144,6 +153,7 @@ BuildGroups(active)
     register char	*q;
     register int	i;
     register unsigned	j;
+    register int	lines;
     int			NGHbuckets;
     char		*fields[5];
 
@@ -164,14 +174,18 @@ BuildGroups(active)
     }
 
     /* Fill in the array. */
+    lines = 0;
     for (p = active, ngp = Groups, i = nGroups; --i >= 0; ngp++, p = q + 1) {
+	lines++;
 	if ((q = strchr(p, '\n')) == NULL) {
-	    (void)fprintf(stderr, "Missing newline near \"%.10s...\"\n", p);
+	    (void)fprintf(stderr, "%s: line %d missing newline\n",
+		    ACTIVE, lines);
 	    exit(1);
 	}
 	*q = '\0';
 	if (EXPsplit(p, ' ', fields, SIZEOF(fields)) != 4) {
-	    (void)fprintf(stderr, "Bad format near \"%.10s...\"\n", p);
+	    (void)fprintf(stderr, "%s: line %d wrong number of fields\n",
+		    ACTIVE, lines);
 	    exit(1);
 	}
 	ngp->Name = fields[0];
@@ -443,6 +457,56 @@ EXPreadfile(F)
 }
 
 
+/*
+**  Handle a newsgroup that isn't in the active file.
+*/
+STATIC NEWSGROUP *
+EXPnotfound(Entry)
+    char		*Entry;
+{
+    static NEWSGROUP	Removeit;
+    register BADGROUP	*bg;
+    register char	*p;
+    struct stat		Sb;
+    char		buff[SPOOLNAMEBUFF];
+
+    /* See if we already know about this group. */
+    for (bg = EXPbadgroups; bg; bg = bg->Next)
+	if (EQ(Entry, bg->Name))
+	    break;
+    if (bg == NULL) {
+	bg = NEW(BADGROUP, 1);
+	bg->Name = COPY(Entry);
+	(void)strcpy(buff, bg->Name);
+	for (p = buff; *p; p++)
+	    if (*p == '.')
+		*p = '/';
+	bg->HasDirectory = stat(buff, &Sb) >= 0 && S_ISDIR(Sb.st_mode);
+	bg->Next = EXPbadgroups;
+	EXPbadgroups = bg;
+	if (!EXPquiet) {
+	    (void)fflush(stdout);
+	    (void)fprintf(stderr, "Group not matched (removed?) %s -- %s\n",
+		Entry,
+		bg->HasDirectory ? "Using default expiration"
+				 : "Purging all articles");
+	}
+    }
+
+    /* Directory still there; use default expiration. */
+    if (bg->HasDirectory)
+	return &EXPdefault;
+
+    /* No directory -- remove it all now. */
+    if (Removeit.Keep == 0) {
+	Removeit.Keep = Now;
+	Removeit.Default = Now;
+	Removeit.Purge = Now;
+    }
+    return &Removeit;
+}
+
+
 /*
 **  Should we keep the specified article?
 */
@@ -461,13 +525,8 @@ EXPkeepit(Entry, when, Expires)
 	return TRUE;
     }
     *p = '\0';
-    if ((ngp = NGfind(Entry)) == NULL) {
-	if (!EXPquiet) {
-	    (void)fflush(stdout);
-	    (void)fprintf(stderr, "Group not matched (removed?) %s\n", Entry);
-	}
-	ngp = &EXPdefault;
-    }
+    if ((ngp = NGfind(Entry)) == NULL)
+	ngp = EXPnotfound(Entry);
     *p = '/';
 
     if (EXPverbose > 2) {
@@ -552,6 +611,7 @@ EXPdoline(out, line, length, arts)
     char		**arts;
 {
     static char		IGNORING[] = "Ignoring bad line, \"%.20s...\"\n";
+    static long		Offset;
     static BUFFER	New;
     register char	*p;
     register char	*q;
@@ -567,6 +627,7 @@ EXPdoline(out, line, length, arts)
     long		size;
     datum		key;
     datum		value;
+    char 		date[20];
 
     /* Split up the major fields. */
     i = EXPsplit(line, HIS_FIELDSEP, fields, SIZEOF(fields));
@@ -609,10 +670,11 @@ EXPdoline(out, line, length, arts)
 	    return TRUE;
 	}
 
-	/* Not time time to forget about this one yet. */
+	/* Not time to forget about this one yet. */
 	if (out) {
-	    where = ftell(out);
+	    where = Offset;
 	    (void)fprintf(out, "%s%c%s\n", fields[0], HIS_FIELDSEP, fields[1]);
+	    Offset += strlen(fields[0]) + 1 + strlen(fields[1]) + 1;
 	    if (EXPverbose > 3)
 		(void)printf("remember: %s\n", line);
 	    EXPhistremember++;
@@ -677,22 +739,27 @@ EXPdoline(out, line, length, arts)
 	    if (EXPsizing && size > 0)
 		EXPsaved += size;
 	    if (EXPremember > 0 && out != NULL) {
-		where = ftell(out);
-		(void)fprintf(out, "%s%c%ld%c%s\n",
+		where = Offset;
+		(void)sprintf(date, "%ld", (long)Arrived);
+		(void)fprintf(out, "%s%c%s%c%s\n",
 			fields[0], HIS_FIELDSEP,
-			(long)Now, HIS_SUBFIELDSEP, HIS_NOEXP);
+			date, HIS_SUBFIELDSEP, HIS_NOEXP);
+		Offset += strlen(fields[0]) + 1
+			+ strlen(date) + 1 + STRLEN(HIS_NOEXP) + 1;
 	        if (EXPverbose > 3)
-		    (void)printf("remember history: %s%c%ld%c%s\n",
+		    (void)printf("remember history: %s%c%s%c%s\n",
 			    fields[0], HIS_FIELDSEP,
-			    (long)Now, HIS_SUBFIELDSEP, HIS_NOEXP);
+			    date, HIS_SUBFIELDSEP, HIS_NOEXP);
 		EXPallgone++;
 	    }
 	}
 	else if (out) {
-	    where = ftell(out);
+	    where = Offset;
 	    (void)fprintf(out, "%s%c%s%c%s\n",
 		    fields[0], HIS_FIELDSEP, fields[1], HIS_FIELDSEP,
 		    New.Data);
+	    Offset += strlen(fields[0]) + 1 + strlen(fields[1]) + 1
+		    + strlen(New.Data) + 1;
 	    if (EXPverbose > 3)
 		(void)printf("remember article: %s%c%s%c%s\n",
 			fields[0], HIS_FIELDSEP, fields[1], HIS_FIELDSEP,
@@ -736,6 +803,8 @@ CleanupAndExit(Server, Paused, x)
     BOOL	Paused;
     int		x;
 {
+    FILE	*F;
+
     if (Server)
 	(void)ICCreserve("");
     if (Paused && ICCgo(EXPreason) != 0) {
@@ -763,6 +832,15 @@ CleanupAndExit(Server, Paused, x)
 	(void)printf("Old entries retained    %8ld\n", EXPhistremember);
     }
 
+    /* Append statistics to a summary file */
+    if (EXPgraph) {
+	F = EXPfopen(FALSE, EXPgraph, "a");
+	(void)fprintf(F, "%ld %ld %ld %ld %ld %ld %ld\n",
+		(long)Now, EXPprocessed, EXPstillhere, EXPallgone,
+		EXPunlinked, EXPhistdrop, EXPhistremember);
+	(void)fclose(F);
+    }
+
     exit(x);
 }
 
@@ -783,7 +861,6 @@ main(ac, av)
     int			ac;
     char		*av[];
 {
-    static char		ACTIVE[] = _PATH_ACTIVE;
     static char		CANTCD[] = "Can't cd to %s, %s\n";
     register int	i;
     register int	line;
@@ -795,6 +872,7 @@ main(ac, av)
     STRING		History;
     STRING		HistoryText;
     STRING		HistoryPath;
+    STRING		HistoryDB;
     char		*Historydir;
     char		*Historypag;
     char		*NHistory;
@@ -815,13 +893,14 @@ main(ac, av)
     IgnoreOld = FALSE;
     History = "history";
     HistoryText = _PATH_HISTORY;
-    HistoryPath = ".";
+    HistoryPath = NULL;
     Writing = TRUE;
     TimeWarp = 0;
     UnlinkFile = FALSE;
+    (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
-    while ((i = getopt(ac, av, "f:h:d:ilnpqr:stv:w:xz:")) != EOF)
+    while ((i = getopt(ac, av, "f:h:d:g:ilnpqr:stv:w:xz:")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -832,6 +911,9 @@ main(ac, av)
 	case 'f':
 	    History = optarg;
 	    break;
+	case 'g':
+	    EXPgraph = optarg;
+	    break;
 	case 'h':
 	    HistoryText = optarg;
 	    break;
@@ -869,7 +951,7 @@ main(ac, av)
 	    Writing = FALSE;
 	    break;
 	case 'z':
-	    EXPunlinkfile = EXPfopen(TRUE, optarg, "w");
+	    EXPunlinkfile = EXPfopen(TRUE, optarg, "a");
 	    UnlinkFile = TRUE;
 	    break;
 	}
@@ -878,9 +960,31 @@ main(ac, av)
     if (ac != 0 && ac != 1)
 	Usage();
 
+    /* Get active file, parse it. */
+    if ((active = ReadInFile(ACTIVE, (struct stat *)NULL)) == NULL) {
+	(void)fprintf(stderr, "Can't read %s, %s\n",
+		ACTIVE, strerror(errno));
+	exit(1);
+    }
+    BuildGroups(active);
+    (void)time(&Now);
+    Now += TimeWarp;
+
+    /* Parse the control file. */
+    if (av[0])
+	F = EQ(av[0], "-") ? stdin : EXPfopen(FALSE, av[0], "r");
+    else
+	F = EXPfopen(FALSE, _PATH_EXPIRECTL, "r");
+    if (!EXPreadfile(F)) {
+	(void)fclose(F);
+	(void)fprintf(stderr, "Format error in control file\n");
+	exit(1);
+    }
+    (void)fclose(F);
+
     /* Set up the link, reserve the lock. */
     if (EXPreason == NULL) {
-	(void)sprintf(buff, "%s %ld", EXPreason, (long)getpid());
+	(void)sprintf(buff, "Expiring process %ld", (long)getpid());
 	EXPreason = COPY(buff);
     }
     if (Server) {
@@ -896,31 +1000,16 @@ main(ac, av)
 	}
     }
 
-    /* Get active file, parse it. */
-    if ((active = ReadInFile(ACTIVE, (struct stat *)NULL)) == NULL) {
-	(void)fprintf(stderr, "Can't read %s, %s\n",
-		ACTIVE, strerror(errno));
-	exit(1);
-    }
-    BuildGroups(active);
-    (void)time(&Now);
-    Now += TimeWarp;
-
-    /* Parse the control file. */
-    F = EXPfopen(FALSE, av[0] ? av[0] : _PATH_EXPIRECTL, "r");
-    if (!EXPreadfile(F)) {
-	(void)fclose(F);
-	(void)fprintf(stderr, "Format error in control file\n");
-	exit(1);
-    }
-    (void)fclose(F);
-
     /* Make the history filenames. */
-    (void)sprintf(buff, "%s.dir", History);
+    HistoryDB = COPY(HistoryText);
+    (void)sprintf(buff, "%s.dir", HistoryDB);
     Historydir = COPY(buff);
-    (void)sprintf(buff, "%s.pag", History);
+    (void)sprintf(buff, "%s.pag", HistoryDB);
     Historypag = COPY(buff);
-    (void)sprintf(buff, "%s/%s.n", HistoryPath, History);
+    if (HistoryPath)
+	(void)sprintf(buff, "%s/%s.n", HistoryPath, History);
+    else
+	(void)sprintf(buff, "%s.n", History);
     NHistory = COPY(buff);
     (void)sprintf(buff, "%s.dir", NHistory);
     NHistorydir = COPY(buff);
@@ -949,7 +1038,7 @@ main(ac, av)
 		exit(1);
 	    }
 	}
-	else if (dbzagain(NHistory, History) < 0) {
+	else if (dbzagain(NHistory, HistoryDB) < 0) {
 	    (void)fprintf(stderr, "Can't dbzagain, %s\n", strerror(errno));
 	    exit(1);
 	}
@@ -971,6 +1060,11 @@ main(ac, av)
 	if ((p = QIOread(qp)) != NULL) {
 	    if (!EXPdoline(out, p, QIOlength(qp), arts)) {
 		Bad = TRUE;
+		if (errno == ENOSPC) {
+		    (void)unlink(NHistory);
+		    (void)unlink(NHistorydir);
+		    (void)unlink(NHistorypag);
+		}
 		break;
 	    }
 	    continue;
@@ -978,13 +1072,13 @@ main(ac, av)
 
 	/* Read or line-format error? */
 	if (QIOerror(qp)) {
-	    (void)fprintf(stderr, "Can't read line %ld, %s\n",
+	    (void)fprintf(stderr, "Can't read line %d, %s\n",
 		    line, strerror(errno));
 	    QIOclose(qp);
 	    CleanupAndExit(Server, Paused, 1);
 	}
 	if (QIOtoolong(qp)) {
-	    (void)fprintf(stderr, "Line %ld too long\n", line);
+	    (void)fprintf(stderr, "Line %d too long\n", line);
 	    QIOclose(qp);
 	    CleanupAndExit(Server, Paused, 1);
 	}
@@ -1034,13 +1128,13 @@ main(ac, av)
 		CleanupAndExit(Server, Paused, 1);
 	    }
 	    /* If user used the -d flag, mark we're done and exit. */
-	    if (!EQ(HistoryPath, ".")) {
+	    if (HistoryPath != NULL) {
 		(void)sprintf(buff, "%s.done", NHistory);
 		(void)fclose(EXPfopen(FALSE, buff, "w"));
 		CleanupAndExit(Server, FALSE, 0);
 	    }
 
-	    if (rename(NHistory, History) < 0
+	    if (rename(NHistory, HistoryText) < 0
 	     || rename(NHistorydir, Historydir) < 0
 	     || rename(NHistorypag, Historypag) < 0) {
 		(void)fprintf(stderr, "Can't replace history files, %s\n",
