@@ -1,4 +1,4 @@
-/*  $Revision: 1.22 $
+/*  $Revision: 1.1 $
 **
 **  A front-end for InterNetNews.
 **  Read UUCP batches and offer them up NNTP-style.  Because we may end
@@ -43,12 +43,18 @@ STATIC HEADER	RequiredHeaders[] = {
     { "Message-ID",	10 },
 #define _messageid	0
     { "Newsgroups",	10 },
+#define _newsgroups	1
     { "From",		 4 },
+#define _from		2
     { "Date",		 4 },
+#define _date		3
     { "Subject",	 7 },
+#define _subject	4
     { "Path",		 4 },
+#define _path		5
 };
 #define IS_MESGID(hp)	((hp) == &RequiredHeaders[_messageid])
+#define IS_PATH(hp)	((hp) == &RequiredHeaders[_path])
 
 
 
@@ -90,10 +96,10 @@ StartChild(fd, path, argv)
     /* Get a child. */
     for (i = 0; (pid = FORK()) < 0; i++) {
 	if (i == MAX_FORKS) {
-	    syslog(L_ERROR, "cant fork %s %m -- spooling");
+	    syslog(L_ERROR, "cant fork %s %m -- spooling", path);
 	    return -1;
 	}
-	syslog(L_NOTICE, "cant fork %s -- waiting");
+	syslog(L_NOTICE, "cant fork %s -- waiting", path);
 	(void)sleep(60);
     }
 
@@ -156,20 +162,15 @@ WaitForChildren(i)
 
 /*
 **  Make a temporary filename that is unlikely to collide with mktemp
-**  or cause problems for sites with 14-character filename limits.
-**  Used to generate names so they would sort in time order, but there
-**  is no real need for that.
+**  or cause problems for sites with 14-character filename limits,
+**  and that hints at the sending host.
 */
 STATIC void
 TempName(dir, buff)
     char	*dir;
     char	*buff;
 {
-    time_t	now;
-
-    (void)time(&now);
-    now &= 0xFFFFFFFF;
-    (void)sprintf(buff, "%s/%08.8lxXXXXXX", dir, (long)now);
+    (void)sprintf(buff, "%s/%.8sXXXXXX", dir, UUCPHost ? UUCPHost : "unknown");
     (void)mktemp(buff);
 }
 
@@ -239,6 +240,12 @@ Process(article)
     register char	*p;
     char		*id;
     char		buff[SMBUF];
+#if	defined(FILE_RNEWS_LOG_DUPS)
+    FILE		*F;
+#endif	/* defined(FILE_RNEWS_LOG_DUPS) */
+#if	!defined(DONT_RNEWS_LOG_DUPS)
+    char		path[40];
+#endif	/* !defined(DONT_RNEWS_LOG_DUPS) */
 
     /* Empty article? */
     if (*article == '\0')
@@ -250,8 +257,18 @@ Process(article)
 	    Reject(article, "bad_article missing %s", hp->Name);
 	    return TRUE;
 	}
-	if (IS_MESGID(hp))
+	if (IS_MESGID(hp)) {
 	    id = p;
+	    continue;
+	}
+#if	!defined(DONT_RNEWS_LOG_DUPS)
+	if (IS_PATH(hp)) {
+	    (void)strncpy(path, p, sizeof path);
+	    path[sizeof path - 1] = '\0';
+	    if ((p = strchr(path, '\n')) != NULL)
+		*p = '\0';
+	}
+#endif	/* !defined(DONT_RNEWS_LOG_DUPS) */
     }
 
     /* Send the NNTP "ihave" message. */
@@ -283,6 +300,17 @@ Process(article)
     case NNTP_SENDIT_VAL:
 	break;
     case NNTP_HAVEIT_VAL:
+#if	defined(SYSLOG_RNEWS_LOG_DUPS)
+	*p = '\0';
+	syslog(L_NOTICE, "duplicate %s %s", id, path);
+#endif	/* defined(SYSLOG_RNEWS_LOG_DUPS) */
+#if	defined(FILE_RNEWS_LOG_DUPS)
+	if ((F = fopen(_PATH_RNEWS_DUP_LOG, "a")) != NULL) {
+	    *p = '\0';
+	    (void)fprintf(stderr, "duplicate %s %s", id, path);
+	    (void)fclose(F);
+	}
+#endif	/* defined(FILE_RNEWS_LOG_DUPS) */
 	return TRUE;
     }
 
@@ -339,6 +367,7 @@ ReadRemainder(fd, first, second)
     register int	used;
     register int	left;
     register int	i;
+    BOOL		ok;
 
     /* Turn the descriptor into a stream. */
     if ((F = fdopen(fd, "r")) == NULL) {
@@ -358,7 +387,7 @@ ReadRemainder(fd, first, second)
     while ((i = fread((POINTER)&article[used], (SIZE_T)1,
 		    (SIZE_T)left, F)) != 0) {
 	if (i < 0) {
-	    syslog(L_FATAL, "cant fread after %ld bytes %m", used);
+	    syslog(L_FATAL, "cant fread after %d bytes %m", used);
 	    exit(1);
 	}
 	used += i;
@@ -374,7 +403,9 @@ ReadRemainder(fd, first, second)
     article[used] = '\0';
     (void)fclose(F);
 
-    return Process(article);
+    ok = Process(article);
+    DISPOSE(article);
+    return ok;
 }
 
 
@@ -579,7 +610,10 @@ Unspool()
     register DIRENTRY	*ep;
     register BOOL	ok;
     struct stat		Sb;
+    char		buff[SMBUF];
+    char		hostname[10];
     int			fd;
+    int			oerrno;
     int			i;
 
     /* Go to the spool directory, get ready to scan it. */
@@ -615,15 +649,31 @@ Unspool()
 	    }
 	    continue;
 	}
+	(void)strncpy(hostname, InputFile, 8);
+	hostname[8] = '\0';
+	UUCPHost = hostname;
 	ok = UnpackOne(&fd, &i);
 	(void)close(fd);
 	WaitForChildren(i);
 
 	if (!ok) {
+	    oerrno = errno;
+	    TempName(_PATH_BADNEWS, buff);
 	    xperror("Unspooling failed");
-	    exit(1);
+	    (void)fprintf(stderr, "Unspooling failed saving to %s %s\n",
+		buff, strerror(errno));
+	    errno = oerrno;
+	    syslog(L_ERROR, "cant unspool saving to %s %m", buff);
+	    if (rename(InputFile, buff) < 0) {
+		xperror(buff);
+		syslog(L_FATAL, "cant rename %s to %s %m", InputFile, buff);
+		exit(1);
+	    }
+	    continue;
 	}
-	(void)unlink(InputFile);
+
+	if (unlink(InputFile) < 0)
+	    syslog(L_ERROR, "cant remove %s %m", InputFile);
     }
     (void)closedir(dp);
 }
@@ -694,11 +744,18 @@ Spool(fd)
 **  NNTP server.
 */
 STATIC BOOL
-OpenRemote(buff)
+OpenRemote(server, buff)
+    char	*server;
     char	*buff;
 {
+    int		i;
+
     /* Open the remote connection. */
-    if (NNTPremoteopen(&FromServer, &ToServer, buff) < 0)
+    if (server)
+	i = NNTPconnect(server, &FromServer, &ToServer, buff);
+    else
+	i = NNTPremoteopen(&FromServer, &ToServer, buff);
+    if (i < 0)
 	return FALSE;
 
     *buff = '\0';
@@ -711,6 +768,25 @@ OpenRemote(buff)
 }
 
 
+/*
+**  Can't connect to server; print message and spool if necessary.
+*/
+STATIC NORETURN
+CantConnect(buff, mode, fd)
+    char	*buff;
+    int		mode;
+    int		fd;
+{
+    if (buff[0])
+	syslog(L_NOTICE, "rejected connection %s", REMclean(buff));
+    else
+	syslog(L_FATAL, "cant open_remote %m");
+    if (mode != 'U')
+	Spool(fd);
+    exit(1);
+}
+
+
 /*
 **  Log an incorrect usage.
 */
@@ -727,28 +803,29 @@ main(ac, av)
     int		ac;
     char	*av[];
 {
-    static char SERVER_REJECT[] = "rejected connection %s";
     int		fd;
     int		i;
     int		mode;
     char	buff[SMBUF];
+    char	*Slave;
 
     /* First thing, set up logging and our identity. */
     openlog("rnews", L_OPENLOG_FLAGS, LOG_INN_PROG);
     if (setgid(getegid()) < 0) {
-	syslog(L_FATAL, "cant setgid to %d %m", getegid());
-	exit(1);
+	syslog(L_ERROR, "cant setgid to %d %m", getegid());
     }
     if (setuid(geteuid()) < 0) {
 	syslog(L_FATAL, "cant setuid to %d %m", geteuid());
 	exit(1);
     }
     UUCPHost = getenv(_ENV_UUCPHOST);
+    (void)umask(NEWSUMASK);
 
     /* Parse JCL. */
     fd = STDIN;
     mode = '\0';
-    while ((i = getopt(ac, av, "h:Uv")) != EOF)
+    Slave = NULL;
+    while ((i = getopt(ac, av, "h:S:Uv")) != EOF)
 	switch (i) {
 	default:
 	    Usage();
@@ -756,6 +833,9 @@ main(ac, av)
 	case 'h':
 	    UUCPHost = *optarg ? optarg : NULL;
 	    break;
+	case 'S':
+	    Slave = optarg;
+	    break;
 	case 'U':
 	    mode = i;
 	    break;
@@ -786,28 +866,26 @@ main(ac, av)
     }
 
     /* Open the link to the server. */
-    if (NNTPlocalopen(&FromServer, &ToServer, buff) < 0) {
-	/* If server rejected us, no point in continuing. */
-	if (buff[0]) {
-	    syslog(L_NOTICE, SERVER_REJECT, REMclean(buff));
-	    if (mode != 'U')
-		Spool(fd);
-	    exit(1);
-	}
-
-	/* Try to open remote link. */
-	if (!OpenRemote(buff)) {
-	    if (buff[0])
-		syslog(L_NOTICE, SERVER_REJECT, REMclean(buff));
-	    else
-		syslog(L_FATAL, "cant open_remote %m");
-	    if (mode != 'U')
-		Spool(fd);
-	    exit(1);
-	}
+    if (Slave) {
+	if (!OpenRemote(Slave, buff))
+	    CantConnect(buff, mode, fd);
     }
-    CloseOnExec((int)fileno(FromServer));
-    CloseOnExec((int)fileno(ToServer));
+    else {
+#if	defined(DO_RNEWSLOCALCONNECT)
+	if (NNTPlocalopen(&FromServer, &ToServer, buff) < 0) {
+	    /* If server rejected us, no point in continuing. */
+	    if (buff[0])
+		CantConnect(buff, mode, fd);
+	    if (!OpenRemote((char *)NULL, buff))
+		CantConnect(buff, mode, fd);
+	}
+#else
+	if (!OpenRemote((char *)NULL, buff))
+	    CantConnect(buff, mode, fd);
+#endif	/* defined(DO_RNEWSLOCALCONNECT) */
+    }
+    CloseOnExec((int)fileno(FromServer), TRUE);
+    CloseOnExec((int)fileno(ToServer), TRUE);
 
     /* Execute the command. */
     if (mode == 'U')
