@@ -1,5 +1,5 @@
 #! /bin/sh
-##  $Revision: 1.2 $
+##  $Revision: 1.1 $
 ##  Perform a test(1) and execute a command string if the test is true;
 ##  otherwise exit 0.  Usage:
 ##	testit <test> <command>
@@ -17,8 +17,7 @@ esac
 TEST="$1"
 COMMAND="$2"
 
-##  Note the lack of quotes around ${TEST} below.
-if test ${TEST} ; then
+if eval "test ${TEST}" ; then
     eval "${COMMAND}"
     exit $?
 fi
