@@ -1,6 +1,91 @@
-/* $Revision: 1.13 $
+/* $Revision: 1.14 $
 **
 **  $Log: patchlevel.h,v $
+**  Revision 1.14  1993/03/18  21:04:15  rsalz
+**  patch04:  Add NEWSUMASK (default 2) and have appropriate programs set it.
+**  patch04:  Add comment about NEWSLIB to config.dist <chongo@ncd.com>
+**  patch04:  Add YACC config variable.
+**  patch04:  Alias FNDELAY to O_NDELAY for systems without it <urlichs@smurf.sub.org>
+**  patch04:  Use $(SHELL) not sh in Makefiles <osm@msen.com>
+**  patch04:  Use "echo ...|su" not "su -c ..." in rc.news and BUILD.
+**  patch04:  Document setsockopt/svr4 problems.
+**  patch04:  overview.fmt.5 referenced makeoverview <henrich@crh.cl.msu.edu>.
+**  patch04:  Various typos in documentation <ry66@rz;uni-karlsruhe.de>,
+**  patch04:  <robert@steffi.demon.co.uk>, <wdh@grouper.mkt.csd.harris.com>.
+**  patch04:  Add sample newsfeeds entry to overchan.8
+**  patch04:  More text for innwatch.ctl, ``make update'', news overview <rsalz>.
+**  patch04:  inews can spool, so you need rnews; update inews.1
+**  patch04:  Add rnews explanations to Install.ms.1
+**  patch04:  Inews should not spool or email if -D given.
+**  patch04:  rnews had bad fopen call <mam@mamunx.garmhausen.de>
+**  patch04:  rnews could not connect to remote server.
+**  patch04:  rnews leaked memory in ReadRemainder.
+**  patch04:  decode unpacked end wrong.
+**  patch04:  decode used bad pointer comparisons.
+**  patch04:  #if was backwards in syslog/syslogd.c <peter@dialix.oz.au>.
+**  patch04:  Remove leading space in filenames in syslog.conf
+**  patch04:  Add -O flag to expireover; fix sorting bug <rob@violet.berkeley.edu>.
+**  patch04:  *** RUN "expireover -a -s" SOON **
+**  patch04:  Malloc overrun in expireover.c article.c <rob@agate.berkeley.edu>
+**  patch04:  Have overchan create needed dirs if overview dir != spool dir.
+**  patch04:  Add overchan .o dependencies to backends/Makefile.
+**  patch04:  avoid unneeded unlink/group in expireover <tale@uunet.uu.net>
+**  patch04:  Add -g to expire <tal@warren.mentorg.com>
+**  patch04:  faster raceless expireover/overchan locking <rob@agate.berkeley.edu>
+**  patch04:  use caseEQ not EQ in CMDmode in nnrpd <urlichs@smurf.sub.org>
+**  patch04:  Two wrong CloseOnExec calls in article.c <stevo@elroy.Jpl.Nasa.Gov>
+**  patch04:  Have nnrpd's HISgetent return (char *) not (STRING).
+**  patch04:  Remove STATIC from nnrpd's CMD_unimp function definition; change
+**  patch04:  it to recognize slave command.
+**  patch04:  Add "date" command (from nntpv2 draft) to nnrpd.
+**  patch04:  Add -u flag and statistics to nntpget.
+**  patch04:  Add -p flag to filechan, buffchan.
+**  patch04:  buffchan shouldn't open dropped sites.
+**  patch04:  Add -A flag to innxmit.
+**  patch04:  newsrequeue re-used variable; coredumped if not logfile mode.
+**  patch04:  expireover and fastrm need <ctype.h>.
+**  patch04:  Move functions in fastrm so STATIC declaration is okay.
+**  patch04:  BUILD and makehistory no longer assume history is in NEWSLIB.
+**  patch04:  Spelled Jon's name wrong in dbz.pch <davison@borland.com>
+**  patch04:  Use memset not bzero in local FD_ZERO macro.
+**  patch04:  getlist parsed positional arguments wrong <chongo@ncd.com>
+**  patch04:  getlist did not send a QUIT to the server.
+**  patch04:  Reverse order of elements in include/uio.h <aej@wpi.WPI.EDU>
+**  patch04:  Have GetFQDN try to force NIS/YP to use DNS <wietse@wzv.win.tue.nl>
+**  patch04:  Fix date parser when hour is 12.
+**  patch04:  Typo in header in send-ihave <mit@huie.hokudai.ac.jp>
+**  patch04:  Had senduucp.log in samples/scanlogs <earle@isolar.Tujunga.CA.US>
+**  patch04:  Have innwatch not complain to console if innd dies <mcooper@usc.edu>
+**  patch04:  Add logwatch into innwatch <Christophe.Wolfhugel@grasp1.univ-lyon1.fr>
+**  patch04:  rmgroup, newgroup, checkgroup are better about updating newsgroups
+**  patch04:  <cs@germany.eu.net>
+**  patch04:  Spurious erroneous mail line in rmgroup.
+**  patch04:  checkgroups mail message is now more clear.
+**  patch04:  Convert remaining scripts in samples to use innshellvars
+**  patch04:  Fix dataloss and fd leak in SITEflush <cjj@sun.com>
+**  patch04:  Don't use strlen on mmap'd active file <bobs@monty.rand.org>
+**  patch04:  Used ModeReason not RejectReason in CCmode, CCreject<dem@meaddata.com>
+**  patch04:  Used wrong argv[] in CCreject <dem@meaddata.com>
+**  patch04:  Don't free NULL pointer in innd/rc.c <peter@dialix.oz.au>.
+**  patch04:  Set all WIP's properly to NULL <dem@meaddata.com>
+**  patch04:  SITEparsefile didn't free old ME entry <dem@meaddata.com>
+**  patch04:  innd had typo in NICE_KIDS #if test <enger@seka.reston.ans.net>
+**  patch04:  Add (void) to setsid call in innd <stevo@elroy.Jpl.Nasa.Gov>
+**  patch04:  SITEwantsgroup didn't check ME patterns
+**  patch04:  <watanabe@argon.material.tohoku.ac.jp>
+**  patch04:  Don't crash if spooling fails <alden@zaphod.mps.ohio-state.edu>
+**  patch04:  Don't reuse socket in innd/cc.c <peter@dialix.oz.au>
+**  patch04:  Protect possible NULL return from RChostname (!?).
+**  patch04:  New NNTP connections to innd must clear any old WIP.
+**  patch04:  Add "p" item to newsfeeds "A" flag; document slave use.
+**  patch04:  rmgroups propagate like newgroups.
+**  patch04:  ctlinnd 'addhist' must open history if server not running.
+**  patch04:  innd clobbered memory on some reloads.
+**  patch04:  Copy SetDescriptorLimit into inndstart.
+**  patch04:  Make LIST case-insensitive in innd.
+**  patch04:  Add hosts.nntp.nolimit
+**  patch04:  Check PID file before innd starts up.
+**
 **  Revision 1.13  1993/01/29  16:51:16  rsalz
 **  patch03:  Don't use absolute paths for true false in BUILD and send-nntp
 **  patch03:  Add G flag to newsfeeds
@@ -282,8 +367,8 @@
 **  Try to avoid changing these.
 */
 #define RELEASE		"1"
-#define PATCHLEVEL	"3"
-#define DATE		"29-Jan-93"
+#define PATCHLEVEL	"4"
+#define DATE		"20-Mar-93"
 
 
 /*
