@@ -1,4 +1,4 @@
-/*  $Revision: 1.24 $
+/*  $Revision: 1.1 $
 **
 **  Routines to read and write the active file.
 */
@@ -31,6 +31,12 @@ STATIC int		ICDactsize;
 	} while (JUSTONCE)
 #define ICDiovrelease(iovp)		DISPOSE((iovp)->iov_base)
 
+#if	defined(MAP_FILE)
+#define MAP__ARG	(MAP_FILE | MAP_SHARED)
+#else
+#define MAP__ARG	(MAP_SHARED)
+#endif	/* defined(MAP_FILE) */
+
 #else
 
 #define ICDiovset(iovp, base, len)	\
@@ -39,6 +45,28 @@ STATIC int		ICDactsize;
 #endif	/* defined(ACT_MMAP) */
 
 
+/*
+**  Close the active file, releasing its resources.
+*/
+STATIC void
+ICDcloseactive()
+{
+    if (ICDactpointer) {
+#if	defined(ACT_MMAP)
+	if (munmap(ICDactpointer, ICDactsize) < 0)
+	    syslog(L_ERROR, "%s cant munmap %m", LogName, ICDactpath);
+#else
+	DISPOSE(ICDactpointer);
+#endif	/* defined(ACT_MMAP) */
+	ICDactpointer = NULL;
+	if (close(ICDactfd) < 0) {
+	    syslog(L_FATAL, "%s cant close %s %m", LogName, ICDactpath);
+	    exit(1);
+	}
+    }
+}
+
+
 /*
 **  Set up the hash and in-core tables.
 */
@@ -123,6 +151,7 @@ ICDwritevactive(vp, vpcount)
     static char		BACKUP[] = _PATH_OLDACTIVE;
     static char		WHEN[] = "backup active";
     register int	fd;
+    char		*dummy;
 
     /* Write the current file to a backup. */
     if (unlink(BACKUP) < 0 && errno != ENOENT) {
@@ -154,8 +183,8 @@ ICDwritevactive(vp, vpcount)
     /* Write it. */
     if (xwritev(fd, vp, vpcount) < 0) {
 	IOError(WHEN);
-	(void)close(fd);
 	syslog(L_ERROR, "%s cant write %s %m", LogName, ICDactpath);
+	(void)close(fd);
 	return FALSE;
     }
 
@@ -171,7 +200,7 @@ ICDwritevactive(vp, vpcount)
     if (Mode != OMrunning) {
 	ICDneedsetup = TRUE;
 	/* Force the active file into memory. */
-	(void)ICDreadactive();
+	(void)ICDreadactive(&dummy);
     }
     else
 	ICDsetup(TRUE);
@@ -225,7 +254,7 @@ ICDnewgroup(Name, Rest)
     BOOL		ret;
 
     /* Set up the scatter/gather vectors. */
-    if (strlen(Name) + strlen(Rest) > SMBUF - 24) {
+    if (strlen(Name) + strlen(Rest) > (SIZE_T)(SMBUF - 24)) {
 	syslog(L_ERROR, "%s too_long %s", LogName, MaxLength(Name, Name));
 	return FALSE;
     }
@@ -285,18 +314,21 @@ ICDrmgroup(ngp)
 **  Open the active file and "map" it into memory.
 */
 char *
-ICDreadactive()
+ICDreadactive(endp)
+    char		**endp;
 {
     struct stat		Sb;
 
-    if (ICDactpointer)
+    if (ICDactpointer) {
+	*endp = ICDactpointer + ICDactsize;
 	return ICDactpointer;
+    }
 
     if ((ICDactfd = open(ICDactpath, O_RDWR)) < 0) {
 	syslog(L_FATAL, "%s cant open %s %m", LogName, ICDactpath);
 	exit(1);
     }
-    CloseOnExec(ICDactfd);
+    CloseOnExec(ICDactfd, TRUE);
 
 #if	defined(ACT_MMAP)
     if (fstat(ICDactfd, &Sb) < 0) {
@@ -306,7 +338,12 @@ ICDreadactive()
     }
     ICDactsize = Sb.st_size;
     ICDactpointer = mmap((caddr_t)0, ICDactsize, PROT_READ|PROT_WRITE,
-			MAP_SHARED, ICDactfd, (off_t)0);
+			MAP__ARG, ICDactfd, (off_t)0);
+    if (ICDactpointer == (char *)-1) {
+	syslog(L_FATAL, "%s cant mmap %d %s %m",
+	    LogName, ICDactfd, ICDactpath);
+	exit(1);
+    }
 
 #else
 
@@ -317,32 +354,11 @@ ICDreadactive()
     ICDactsize = Sb.st_size;
 #endif	/* defined(ACT_MMAP) */
 
+    *endp = ICDactpointer + ICDactsize;
     return ICDactpointer;
 }
 
 
-/*
-**  Close the active file, releasing its resources.
-*/
-void
-ICDcloseactive()
-{
-    if (ICDactpointer) {
-#if	defined(ACT_MMAP)
-	if (munmap(ICDactpointer, ICDactsize) < 0)
-	    syslog(L_ERROR, "%s cant munmap %m", LogName, ICDactpath);
-#else
-	DISPOSE(ICDactpointer);
-#endif	/* defined(ACT_MMAP) */
-	ICDactpointer = NULL;
-	if (close(ICDactfd) < 0) {
-	    syslog(L_FATAL, "%s cant close %s %m", LogName, ICDactpath);
-	    exit(1);
-	}
-    }
-}
-
-
 /*
 **  Write the active file out.
 */
@@ -350,7 +366,7 @@ void
 ICDwriteactive()
 {
 #if	defined(ACT_MMAP)
-    /* No-op. */
+    msync(ICDactpointer, ICDactsize);
 
 #else
 
