@@ -1,4 +1,4 @@
-/*  $Revision: 1.29 $
+/*  $Revision: 1.1 $
 **
 **  Routines for the remote connect channel.  Create an Internet stream socket
 **  that processes connect to.  If the incoming site is not one of our feeds,
@@ -8,7 +8,7 @@
 #include <netdb.h>
 
 
-#if	!defined(NETSWAP)
+#if	!defined(NETSWAP) && !defined(__bsdi__)
 #if	!defined(htons)
 extern unsigned short	htons();
 #endif	/* !defined(htons) */
@@ -27,6 +27,7 @@ typedef struct _REMOTEHOST {
     char	*Name;
     INADDR	Address;
     char	*Password;
+    char	**Patterns;
 } REMOTEHOST;
 
 STATIC INADDR		*RCmaster;
@@ -35,10 +36,11 @@ STATIC char		*RCslaveflag;
 STATIC char		RCnnrpd[] = _PATH_NNRPD;
 STATIC char		RCnnrqd[] = _PATH_NNQRD;
 STATIC char		RCnntpd[] = _PATH_NNTPD;
-STATIC char		RChostpath[] = _PATH_INNDHOSTS;
 STATIC CHANNEL		*RCchan;
 STATIC REMOTEHOST	*RCpeerlist;
 STATIC int		RCnpeerlist;
+STATIC REMOTEHOST	*RCnolimitlist;
+STATIC int		RCnnolimitlist;
 
 
 /*
@@ -70,6 +72,24 @@ RCauthorized(cp, pass)
 }
 
 
+/*
+**  See if a host is in the "nolimit" file.
+*/
+BOOL
+RCnolimit(cp)
+    register CHANNEL	*cp;
+{
+    register REMOTEHOST	*rp;
+    register int	i;
+
+    for (rp = RCnolimitlist, i = RCnnolimitlist; --i >= 0; rp++)
+	/* SUPPRESS 112 *//* Retrieving long where char is stored */
+	if (cp->Address.s_addr == rp->Address.s_addr)
+	    return TRUE;
+    return FALSE;
+}
+
+
 /*
 **  Is this an address of the master?
 */
@@ -203,8 +223,11 @@ RCwritedone()
 **  first element of the address list in modern systems, while it's a field
 **  name in old ones.
 */
-void
-RCreadlist()
+STATIC void
+RCreadfile(list, count, filename)
+    REMOTEHOST		**list;
+    int			*count;
+    char		*filename;
 {
     static char		NOPASS[] = "";
     char		buff[SMBUF];
@@ -215,34 +238,43 @@ RCreadlist()
     register REMOTEHOST	*rp;
     register int	j;
     char		*pass;
+    char		*pats;
     int			errors;
 
     /* Free anything that might have been there. */
-    if (RCpeerlist) {
-	for (rp = RCpeerlist, i = RCnpeerlist; --i >= 0; rp++) {
+    if (*list) {
+	for (rp = *list, i = *count; --i >= 0; rp++) {
 	    DISPOSE(rp->Name);
 	    DISPOSE(rp->Password);
+	    if (rp->Patterns)
+		DISPOSE(rp->Patterns);
 	}
-	DISPOSE(RCpeerlist);
-	RCpeerlist = NULL;
-	RCnpeerlist = 0;
+	DISPOSE(*list);
+	*list = NULL;
+	*count = 0;
     }
 
     /* Open the server file, count the lines. */
-    if ((F = fopen(RChostpath, "r")) == NULL) {
-	syslog(L_FATAL, "%s cant read %s %m", LogName, RChostpath);
+    if ((F = fopen(filename, "r")) == NULL) {
+	syslog(L_FATAL, "%s cant read %s %m", LogName, filename);
 	exit(1);
     }
-    for (i = 0; fgets(buff, sizeof buff, F) != NULL; )
+    for (i = 1; fgets(buff, sizeof buff, F) != NULL; )
 	if (buff[0] != COMMENT_CHAR && buff[0] != '\n')
 	    i++;
-    RCnpeerlist = i;
-    RCpeerlist = NEW(REMOTEHOST, RCnpeerlist);
+    *count = i;
+    rp = *list = NEW(REMOTEHOST, *count);
+#if	!defined(DO_HAVE_UNIX_DOMAIN)
+    rp->Address.s_addr = inet_addr(LOOPBACK_HOST);
+    rp->Name = COPY("localhost");
+    rp->Password = COPY(NOPASS);
+    rp->Patterns = NULL;
+    rp++;
+#endif	/* !defined(DO_HAVE_UNIX_DOMAIN) */
 
     /* Now read the file to add all the hosts. */
     (void)fseek(F, (OFFSET_T)0, SEEK_SET);
-    errors = 0;
-    for (rp = RCpeerlist; fgets(buff, sizeof buff, F) != NULL; ) {
+    for (errors = 0; fgets(buff, sizeof buff, F) != NULL; ) {
 	/* Ignore blank and comment lines. */
 	if ((p = strchr(buff, '\n')) != NULL)
 	    *p = '\0';
@@ -250,15 +282,23 @@ RCreadlist()
 	    *p = '\0';
 	if (buff[0] == '\0')
 	    continue;
-	if ((pass = strchr(buff, ':')) != NULL)
+	if ((pass = strchr(buff, ':')) != NULL) {
 	    *pass++ = '\0';
-	else
+	    if ((pats = strchr(pass, ':')) != NULL)
+		*pats++ = '\0';
+	    else
+		pats = NULL;
+	}
+	else {
 	    pass = NOPASS;
+	    pats = NULL;
+	}
 
 	/* Was host specified as as dotted quad? */
 	if ((rp->Address.s_addr = inet_addr(buff)) != (unsigned long)-1) {
 	    rp->Name = COPY(buff);
 	    rp->Password = COPY(pass);
+	    rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
 	    rp++;
 	    continue;
 	}
@@ -284,21 +324,23 @@ RCreadlist()
 	    COPYADDR(&rp->Address, hp->h_addr_list[0]);
 	    rp->Name = COPY(hp->h_name);
 	    rp->Password = COPY(pass);
+	    rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
 	    rp++;
 	    continue;
 	}
 
 	/* Note the relative position, grow the array, and restore it. */
-	j = rp - RCpeerlist;
-	RCnpeerlist += i - 1;
-	RENEW(RCpeerlist, REMOTEHOST, RCnpeerlist);
-	rp = &RCpeerlist[j];
+	j = rp - *list;
+	*count += i - 1;
+	RENEW(*list, REMOTEHOST, *count);
+	rp = *list + j;
 
 	/* Add all the hosts. */
 	for (i = 0; hp->h_addr_list[i]; i++) {
 	    COPYADDR(&rp->Address, hp->h_addr_list[i]);
 	    rp->Name = COPY(hp->h_name);
 	    rp->Password = COPY(pass);
+	    rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
 	    rp++;
 	}
 #else
@@ -306,18 +348,35 @@ RCreadlist()
 	COPYADDR(&rp->Address, hp->h_addr);
 	rp->Name = COPY(hp->h_name);
 	rp->Password = COPY(pass);
+	rp->Patterns = pats ? CommaSplit(COPY(pats)) : NULL;
 	rp++;
 #endif	/* defined(h_addr) */
     }
+    *count = rp - *list;
 
     if (fclose(F) == EOF)
-	syslog(L_ERROR, "%s cant fclose %s %m", LogName, RChostpath);
+	syslog(L_ERROR, "%s cant fclose %s %m", LogName, filename);
 
     if (errors)
-	syslog(L_ERROR, "%s bad_hosts %d in %s", LogName, errors, RChostpath);
+	syslog(L_ERROR, "%s bad_hosts %d in %s", LogName, errors, filename);
 }
 
 
+void
+RCreadlist()
+{
+    static char	INNDHOSTS[] = _PATH_INNDHOSTS;
+    char	name[sizeof _PATH_INNDHOSTS + sizeof ".nolimit"];
+    struct stat	Sb;
+
+    RCreadfile(&RCpeerlist, &RCnpeerlist, INNDHOSTS);
+    FileGlue(name, INNDHOSTS, '.', "nolimit");
+    if (stat(name, &Sb) >= 0)
+	RCreadfile(&RCnolimitlist, &RCnnolimitlist, name);
+}
+
+
+
 /*
 **  Find the name of a remote host we've connected to.
 */
@@ -338,6 +397,40 @@ RChostname(cp)
 }
 
 
+/*
+**  Is the remote site allowed to post to this group?
+*/
+BOOL
+RCcanpost(cp, group)
+    register CHANNEL	*cp;
+    register char	*group;
+{
+    register REMOTEHOST	*rp;
+    register BOOL	match;
+    register BOOL	subvalue;
+    register char	**argv;
+    register char	*pat;
+    register int	i;
+
+    for (rp = RCpeerlist, i = RCnpeerlist; --i >= 0; rp++) {
+	/* SUPPRESS 112 *//* Retrieving long where char is stored */
+	if (cp->Address.s_addr != rp->Address.s_addr)
+	    continue;
+	if (rp->Patterns == NULL)
+	    break;
+	for (match = TRUE, argv = rp->Patterns; (pat = *argv++) != NULL; ) {
+	    subvalue = *pat != SUB_NEGATE;
+	    if (!subvalue)
+		pat++;
+	    if (wildmat(group, pat))
+		match = subvalue;
+	}
+	return match;
+    }
+    return TRUE;
+}
+
+
 /*
 **  Create the channel.
 */
@@ -397,30 +490,30 @@ RCsetup(i, master)
 	    RCmaster = NEW(INADDR, 1);
 	    COPYADDR(&RCmaster[0], &a);
 	}
-
-	/* Must be a text name. */
-	if ((hp = gethostbyname(master)) == NULL) {
-	    syslog(L_FATAL, "%s cant gethostbyname %s %m", LogName, master);
-	    exit(1);
-	}
-
+	else {
+	    /* Must be a text name. */
+	    if ((hp = gethostbyname(master)) == NULL) {
+		syslog(L_FATAL, "%s cant gethostbyname %s %m", LogName, master);
+		exit(1);
+	    }
 #if	defined(h_addr)
-	/* Count the addresses. */
-	for (i = 0; hp->h_addr_list[i]; i++)
-	    continue;
-	if (i == 0) {
-	    syslog(L_FATAL, "%s no_address %s %m", LogName, master);
-	    exit(1);
-	}
-	RCnmaster = i;
-	RCmaster = NEW(INADDR, RCnmaster);
-	for (i = 0; hp->h_addr_list[i]; i++)
-	    COPYADDR(&RCmaster[i], hp->h_addr_list[i]);
+	    /* Count the addresses. */
+	    for (i = 0; hp->h_addr_list[i]; i++)
+		continue;
+	    if (i == 0) {
+		syslog(L_FATAL, "%s no_address %s %m", LogName, master);
+		exit(1);
+	    }
+	    RCnmaster = i;
+	    RCmaster = NEW(INADDR, RCnmaster);
+	    for (i = 0; hp->h_addr_list[i]; i++)
+		COPYADDR(&RCmaster[i], hp->h_addr_list[i]);
 #else
-	RCnmaster = 1;
-	RCmaster = NEW(INADDR, 1)
-	COPYADDR(&RCmaster[0], &a);
+	    RCnmaster = 1;
+	    RCmaster = NEW(INADDR, 1)
+	    COPYADDR(&RCmaster[0], &a);
 #endif	/* defined(h_addr) */
+	}
 
 	/* Set flag for nnrp. */
 	(void)sprintf(buff, "-S%s", master);
@@ -442,8 +535,10 @@ RCclose()
     RCchan = NULL;
     if (RCpeerlist) {
 	for (rp = RCpeerlist, i = RCnpeerlist; --i >= 0; rp++) {
-	    DISPOSE(rp->Password);
 	    DISPOSE(rp->Name);
+	    DISPOSE(rp->Password);
+	    if (rp->Patterns)
+		DISPOSE(rp->Patterns);
 	}
 	DISPOSE(RCpeerlist);
 	RCpeerlist = NULL;
