@@ -1,11 +1,13 @@
-/*  $Revision: 1.13 $
+/*  $Revision: 1.1 $
 **
 */
 #include <stdio.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#if	defined(DO_HAVE_UNIX_DOMAIN)
 #include <sys/un.h>
+#endif	/* defined(DO_HAVE_UNIX_DOMAIN) */
 #include <netinet/in.h>
 #include <netdb.h>
 #include "configdata.h"
@@ -15,7 +17,7 @@
 #include "clibrary.h"
 
 
-#if	!defined(ntohs) && !defined(NETSWAP)
+#if	!defined(ntohs) && !defined(NETSWAP) && !defined(__bsdi__)
 extern unsigned short	ntohs();
 #endif	/* !defined(ntohs) && !defined(NETSWAP) */
 
@@ -30,6 +32,25 @@ NNTPconnect(p, FromServerp, ToServerp, errbuff)
     FILE		**FromServerp;
     FILE		**ToServerp;
     char		*errbuff;
+{
+    char		buf[25];
+
+    (void) sprintf(buf, "%d", NNTP_PORT);
+    return NNTPconnectPort(p, FromServerp, ToServerp, errbuff, buf);
+}
+
+/*
+**  Open a connection to an NNTP server and create stdio FILE's for talking
+**  to it.  Return -1 on error.  This one allows the remote port("service")
+**  to be specified.
+*/
+int
+NNTPconnectPort(p, FromServerp, ToServerp, errbuff, port)
+    register char	*p;
+    FILE		**FromServerp;
+    FILE		**ToServerp;
+    char		*errbuff;
+    char		*port;
 {
     char		**ap;
     char		*fakelist[2];
@@ -44,39 +65,70 @@ NNTPconnect(p, FromServerp, ToServerp, errbuff)
     struct hostent	fakehp;
     struct in_addr	quadaddr;
     struct sockaddr_in	server;
+    struct servent	*sp;
+    int			portnum, protonum;
+    char		*proto, *tp;
 
     buff = errbuff ? errbuff : mybuff;
     *buff = '\0';
-    quadaddr.s_addr = inet_addr(p);
-    if (quadaddr.s_addr != (unsigned long)-1) {
-	/* Host was specified as a dotted-quad internet address.  Fill in
-	 * the parts of the hostent struct that we need. */
-	fakehp.h_length = sizeof quadaddr;
-	fakehp.h_addrtype = AF_INET;
-	hp = &fakehp;
-	fakelist[0] = (char *)&quadaddr;
-	fakelist[1] = NULL;
-	ap = fakelist;
+
+    if ((tp = strchr(port, '/')) != NULL) {
+	proto = tp+1;
+	*tp = '\0';		/* XXX - icky, modifies caller's vars */
+    } else {
+	proto = "tcp";
     }
-    else if ((hp = gethostbyname(p)) != NULL) {
-	/* Symbolic host name. */
+    portnum = htons(atoi(port));
+    if (!portnum) {
+	if ((sp = getservbyname(port, proto)) != NULL) {
+	    portnum = sp->s_port;
+        } else {
+	    return -1;
+        }
+    }
+
+    if (!strcmp("tcp", proto)) {
+	if (inet_aton(p, &quadaddr)) {
+	    /* Host was specified as a dotted-quad internet address.  Fill in
+	     * the parts of the hostent struct that we need. */
+	    fakehp.h_length = sizeof quadaddr;
+	    fakehp.h_addrtype = AF_INET;
+	    hp = &fakehp;
+	    fakelist[0] = (char *)&quadaddr;
+	    fakelist[1] = NULL;
+	    ap = fakelist;
+	}
+	else if ((hp = gethostbyname(p)) != NULL) {
+	    /* Symbolic host name. */
 #if	defined(h_addr)
-	ap = hp->h_addr_list;
+	    ap = hp->h_addr_list;
 #else
-	/* Fake up an address list for old systems. */
-	fakelist[0] = (char *)hp->h_addr;
-	fakelist[1] = NULL;
-	ap = fakelist;
+	    /* Fake up an address list for old systems. */
+	    fakelist[0] = (char *)hp->h_addr;
+	    fakelist[1] = NULL;
+	    ap = fakelist;
 #endif	/* defined(h_addr) */
+	}
+	else {
+	    /* Not a host name. */
+	    return -1;
+	}
+    } /*tcp*/
+    /*
+    else if (!strcmp("dnet", proto)) {
+	... decnet support would go here (as an example)
+	... clone this for XNS, or other supported link layers
     }
-    else
-	/* Not a host name. */
+    */
+    else {
+	errno = EAFNOSUPPORT;
 	return -1;
+    }
 
     /* Set up the socket address. */
     (void)memset((POINTER)&server, 0, sizeof server);
     server.sin_family = hp->h_addrtype;
-    server.sin_port = ntohs(NNTP_PORT);
+    server.sin_port = portnum;
 
     /* Loop through the address list, trying to connect. */
     for (; ap && *ap; ap++) {
