@@ -1,4 +1,4 @@
-/*  $Revision: 1.12 $
+/*  $Revision: 1.1 $
 **
 **  Net News Reading Protocol server.
 */
@@ -9,6 +9,12 @@
 #include <errno.h>
 #include <ctype.h>
 #include <sys/file.h>
+#if	defined(VAR_VARARGS)
+#include <varargs.h>
+#endif	/* defined(VAR_VARARGS) */
+#if	defined(VAR_STDARGS)
+#include <stdarg.h>
+#endif	/* defined(VAR_STDARGS) */
 #include <syslog.h>
 #include <fcntl.h>
 #include <sys/socket.h>
@@ -22,6 +28,13 @@
 #include "macros.h"
 
 
+/*
+**  Maximum input line length, sigh.
+*/
+#define ART_LINE_LENGTH		1000
+#define ART_LINE_MALLOC		1024
+
+
 /*
 **  Some convenient shorthands.
 */
@@ -33,7 +46,8 @@ typedef struct in_addr	INADDR;
 
 
 /*
-**  The XTHREAD command is too ugly to talk about.
+**  The XTHREAD command is too ugly to talk about, but if you want it
+**  change DONT_DO_XTRHEAD to DO_DO_XTRHEAD, below.
 */
 #define DONT_DO_XTHREAD
 #define THREAD_NAMES_FLAT
@@ -48,11 +62,22 @@ typedef struct in_addr	INADDR;
 */
 typedef struct _GROUPENTRY {
     char	*Name;
+    ARTNUM	High;
+    ARTNUM	Low;
     char	Flag;
     char	*Alias;
 } GROUPENTRY;
 
 
+/*
+**  A range of article numbers.
+*/
+typedef struct _ARTRANGE {
+    int		Low;
+    int		High;
+} ARTRANGE;
+
+
 /*
 **  What READline returns.
 */
@@ -87,26 +112,32 @@ extern char	NOACCESS[];
 EXTERN char	PERMpass[20];
 EXTERN char	PERMuser[20];
 EXTERN char	*RemoteMaster;
-EXTERN int	*ARTnumbers;
+EXTERN ARTNUM	*ARTcache;
+EXTERN ARTNUM	*ARTnumbers;
 EXTERN int	ARTindex;
 EXTERN int	ARTsize;
 extern int	PERMdefault;
 EXTERN long	ARTcount;
 EXTERN long	GRParticles;
 EXTERN long	GRPcount;
+EXTERN char	GRPlast[SPOOLNAMEBUFF];
 EXTERN long	POSTreceived;
 EXTERN long	POSTrejected;
 
 
+#if	NNRP_LOADLIMIT > 0
+extern int		GetLoadAverage();
+#endif	/* NNRP_LOADLIMIT > 0 */
 extern STRING		ARTpost();
 extern void		ARTclose();
+extern void		ARTreadschema();
 extern char		*Glom();
 extern int		Argify();
-extern void		ExitWithStats();
+extern NORETURN		ExitWithStats();
 extern BOOL		GetGroupList();
 extern void		GRPreport();
 extern GROUPENTRY	*GRPfind();
-extern STRING		HISgetent();
+extern char		*HISgetent();
 extern long		LOCALtoGMT();
 extern BOOL		NGgetlist();
 extern long		NNTPtoGMT();
@@ -115,8 +146,10 @@ extern BOOL		PERMinfile();
 extern BOOL		PERMmatch();
 extern BOOL		ParseDistlist();
 extern READTYPE		READline();
+extern void		OVERclose();
 #if	defined(VAR_STDARGS)
 extern void		Reply(char *, ...);
-#else
-extern void		Reply();
 #endif	/* defined(VAR_STDARGS) */
+#if	defined(VAR_VARARGS)
+extern void		Reply();
+#endif	/* defined(VAR_VARARGS) */
